searchData={"items":[{"type":"module","title":"ModularInverse","doc":"A library for calculating modular multiplicative inverses in Elixir.\n\nThe modular multiplicative inverse of an integer `a` modulo `m` is an integer `x`\nsuch that `(a * x) mod m = 1`. This inverse exists if and only if `a` and `m`\nare coprime (i.e., their greatest common divisor is 1).","ref":"ModularInverse.html"},{"type":"module","title":"Examples - ModularInverse","doc":"iex> ModularInverse.of(3, 11)\n    4\n    iex> rem(3 * 4, 11)\n    1\n\n    iex> ModularInverse.of(7, 13)\n    2\n    iex> rem(7 * 2, 13)\n    1","ref":"ModularInverse.html#module-examples"},{"type":"module","title":"Error Cases - ModularInverse","doc":"When the modular inverse does not exist (i.e., when `a` and `m` are not coprime),\nthe function raises a `RuntimeError`:\n\n    iex> ModularInverse.of(4, 8)\n    ** (RuntimeError) The inverse of the given arguments does not exist, since they are not mutually prime.","ref":"ModularInverse.html#module-error-cases"},{"type":"module","title":"Algorithm - ModularInverse","doc":"This implementation uses the Extended Euclidean Algorithm to find the modular inverse.\nThe algorithm efficiently computes the greatest common divisor and the Bézout coefficients,\nwhich are used to construct the modular inverse.","ref":"ModularInverse.html#module-algorithm"},{"type":"module","title":"Performance - ModularInverse","doc":"The time complexity is O(log min(a, m)), making it efficient for large numbers.","ref":"ModularInverse.html#module-performance"},{"type":"function","title":"ModularInverse.of/2","doc":"Calculates the modular multiplicative inverse of `a` modulo `m`.\n\nReturns the smallest non-negative integer `x` such that `(a * x) mod m = 1`.","ref":"ModularInverse.html#of/2"},{"type":"function","title":"Parameters - ModularInverse.of/2","doc":"- `a` - The integer for which to find the modular inverse\n  - `m` - The modulus (must be positive)","ref":"ModularInverse.html#of/2-parameters"},{"type":"function","title":"Returns - ModularInverse.of/2","doc":"- `non_neg_integer()` - The modular multiplicative inverse of `a` modulo `m`","ref":"ModularInverse.html#of/2-returns"},{"type":"function","title":"Raises - ModularInverse.of/2","doc":"- `RuntimeError` - When the modular inverse does not exist (when `a` and `m` are not coprime)","ref":"ModularInverse.html#of/2-raises"},{"type":"function","title":"Examples - ModularInverse.of/2","doc":"iex> ModularInverse.of(3, 11)\n    4\n    iex> ModularInverse.of(7, 13)\n    2\n    iex> ModularInverse.of(1, 5)\n    1","ref":"ModularInverse.html#of/2-examples"},{"type":"function","title":"Mathematical Background - ModularInverse.of/2","doc":"The modular multiplicative inverse exists if and only if `a` and `m` are coprime.\nWhen it exists, it is unique modulo `m`. The function returns the smallest\nnon-negative representative of this inverse.\n\nFor example, with `a = 3` and `m = 11`:\n- The inverse is `4` because `(3 * 4) mod 11 = 12 mod 11 = 1`\n- Other valid inverses would be `4 + 11k` for any integer `k`, but `4` is the smallest non-negative one.","ref":"ModularInverse.html#of/2-mathematical-background"},{"type":"extras","title":"ModularInverse","doc":"# ModularInverse\n\nA fast and efficient Elixir library for calculating modular multiplicative inverses.\n\nThe modular multiplicative inverse of an integer `a` modulo `m` is an integer `x` such that `(a * x) mod m = 1`. This library provides a robust implementation using the Extended Euclidean Algorithm.","ref":"readme.html"},{"type":"extras","title":"Installation - ModularInverse","doc":"The package can be installed\nby adding `modular_inverse` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:modular_inverse, \"~> 1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - ModularInverse","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Basic Usage - ModularInverse","doc":"```elixir\n# Calculate the modular inverse of 3 modulo 11\niex> ModularInverse.of(3, 11)\n4\n\n# Verify the result: (3 * 4) mod 11 = 12 mod 11 = 1\niex> rem(3 * 4, 11)\n1\n\n# Another example\niex> ModularInverse.of(7, 13)\n2\niex> rem(7 * 2, 13)\n1\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Error Handling - ModularInverse","doc":"When the modular inverse does not exist (i.e., when `a` and `m` are not coprime), the function raises a `RuntimeError`:\n\n```elixir\niex> ModularInverse.of(4, 8)\n** (RuntimeError) The inverse of the given arguments does not exist, since they are not mutually prime.\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"Mathematical Background - ModularInverse","doc":"","ref":"readme.html#mathematical-background"},{"type":"extras","title":"What is a Modular Multiplicative Inverse? - ModularInverse","doc":"Given integers `a` and `m`, the modular multiplicative inverse of `a` modulo `m` is an integer `x` such that:\n\n```\n(a * x) mod m = 1\n```","ref":"readme.html#what-is-a-modular-multiplicative-inverse"},{"type":"extras","title":"When Does It Exist? - ModularInverse","doc":"The modular multiplicative inverse exists if and only if `a` and `m` are coprime (i.e., their greatest common divisor is 1).","ref":"readme.html#when-does-it-exist"},{"type":"extras","title":"Examples - ModularInverse","doc":"- For `a = 3` and `m = 11`: The inverse is `4` because `(3 * 4) mod 11 = 12 mod 11 = 1`\n- For `a = 7` and `m = 13`: The inverse is `2` because `(7 * 2) mod 13 = 14 mod 13 = 1`\n- For `a = 4` and `m = 8`: No inverse exists because `gcd(4, 8) = 4 ≠ 1`","ref":"readme.html#examples"},{"type":"extras","title":"Algorithm - ModularInverse","doc":"This library uses the Extended Euclidean Algorithm to efficiently compute modular inverses. The algorithm:\n\n1. Computes the greatest common divisor (GCD) of `a` and `m`\n2. If the GCD is 1, calculates the Bézout coefficients\n3. Uses the coefficients to construct the modular inverse\n4. Normalizes the result to the smallest non-negative representative","ref":"readme.html#algorithm"},{"type":"extras","title":"Performance - ModularInverse","doc":"- **Time Complexity**: O(log min(a, m))\n- **Space Complexity**: O(1)\n\nThis makes it efficient even for very large numbers.","ref":"readme.html#performance"},{"type":"extras","title":"API Reference - ModularInverse","doc":"### `ModularInverse.of(a, m)`\n\nCalculates the modular multiplicative inverse of `a` modulo `m`.\n\n#### Parameters\n\n- `a` - The integer for which to find the modular inverse\n- `m` - The modulus (must be positive)\n\n#### Returns\n\n- `non_neg_integer()` - The modular multiplicative inverse of `a` modulo `m`\n\n#### Raises\n\n- `RuntimeError` - When the modular inverse does not exist (when `a` and `m` are not coprime)\n\n#### Examples\n\n```elixir\niex> ModularInverse.of(3, 11)\n4\n\niex> ModularInverse.of(7, 13)\n2\n\niex> ModularInverse.of(1, 5)\n1\n```","ref":"readme.html#api-reference"},{"type":"extras","title":"Applications - ModularInverse","doc":"Modular multiplicative inverses are fundamental in many areas of mathematics and computer science:\n\n- **Cryptography**: RSA encryption, elliptic curve cryptography\n- **Number Theory**: Solving linear congruences\n- **Computer Science**: Hash functions, checksums\n- **Algebra**: Working in finite fields","ref":"readme.html#applications"},{"type":"extras","title":"Testing - ModularInverse","doc":"The library includes comprehensive randomized testing to ensure correctness across a wide range of inputs. The test suite validates the mathematical property that `(a * inverse) mod m = 1` for all valid inputs.","ref":"readme.html#testing"},{"type":"extras","title":"Contributing - ModularInverse","doc":"Contributions are welcome! Please feel free to submit a Pull Request. For major changes, please open an issue first to discuss what you would like to change.","ref":"readme.html#contributing"},{"type":"extras","title":"License - ModularInverse","doc":"Copyright (c) 2025 University of Kitakyushu\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#license"},{"type":"extras","title":"Documentation - ModularInverse","doc":"Documentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#documentation"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [1.0.0] - 2025-7-18","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- **Core functionality**: `ModularInverse.of/2` function for calculating modular multiplicative inverses\n- **Algorithm implementation**: Extended Euclidean Algorithm for efficient computation\n- **Error handling**: Proper `RuntimeError` for cases where inverse doesn't exist\n- **Type specifications**: Complete type specs for all functions\n- **Comprehensive testing**: Randomized test suite with 10,000 test cases\n- **Development tooling**: Credo, Dialyzer, and Spellweaver integration\n- **Documentation**: Complete API documentation with examples and mathematical background\n- **Project configuration**: Full mix.exs setup with dependencies and build aliases\n- Comprehensive documentation with mathematical background\n- Extended Euclidean Algorithm implementation\n- Randomized testing with 10,000 test cases\n- Type specifications for all functions\n- Error handling for non-coprime numbers\n- Project structure with development tooling\n- Test coverage and validation","ref":"changelog.html#added"},{"type":"extras","title":"Features - Changelog","doc":"- **Modular Inverse Calculation**: Calculate the modular multiplicative inverse of any integer modulo another\n- **Performance**: O(log min(a, m)) time complexity for efficient computation with large numbers\n- **Mathematical Accuracy**: Handles edge cases and validates mathematical properties\n- **Error Handling**: Clear error messages when inverse doesn't exist\n- **Type Safety**: Complete type specifications for compile-time safety","ref":"changelog.html#features"},{"type":"extras","title":"Technical Details - Changelog","doc":"- **Algorithm**: Uses Extended Euclidean Algorithm for optimal performance\n- **Normalization**: Returns smallest non-negative representative of the inverse\n- **Validation**: Comprehensive test suite validates `(a * inverse) mod m = 1`\n- **Dependencies**: Built on `BinaryExtendedGcd` for core algorithm\n- **Tooling**: Integrated with modern Elixir development tools","ref":"changelog.html#technical-details"},{"type":"extras","title":"Documentation - Changelog","doc":"- **Module Documentation**: Complete mathematical background and usage examples\n- **Function Documentation**: Detailed parameter descriptions and return types\n- **README**: Comprehensive guide with installation, usage, and applications\n- **API Reference**: Complete function documentation with examples\n- **Mathematical Background**: Clear explanation of modular multiplicative inverses","ref":"changelog.html#documentation"},{"type":"extras","title":"Development - Changelog","doc":"- **Code Quality**: Credo integration for code analysis\n- **Type Checking**: Dialyzer integration for static analysis\n- **Spell Checking**: Spellweaver integration for documentation quality\n- **Testing**: Comprehensive randomized test suite\n- **Build System**: Mix aliases for quality checks and documentation\n\n---","ref":"changelog.html#development"},{"type":"extras","title":"Version History - Changelog","doc":"- **1.0.0**: First stable release with complete modular inverse functionality","ref":"changelog.html#version-history"},{"type":"extras","title":"Contributing - Changelog","doc":"When contributing to this project, please update the changelog by adding a new entry under the [Unreleased] section. Follow the existing format and include:\n\n- **Added**: New features\n- **Changed**: Changes in existing functionality\n- **Deprecated**: Features that will be removed in upcoming releases\n- **Removed**: Features that have been removed\n- **Fixed**: Bug fixes\n- **Security**: Vulnerability fixes","ref":"changelog.html#contributing"},{"type":"extras","title":"Links - Changelog","doc":"- [GitHub Repository](https://github.com/zacky1972/modular_inverse)\n- [Documentation](https://hexdocs.pm/modular_inverse)\n- [Hex Package](https://hex.pm/packages/modular_inverse)","ref":"changelog.html#links"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}